#!/usr/bin/env python
"""
Provides all epoch functions
"""
import json
import requests
from .environment import BASE_TIMEOUT, LIMIT_TIMEOUT


def get_epoch_info(self, epoch_no=None):
    """
    Get the epoch information, all epochs if no epoch specified.

    :param int epoch_no: epoch number to fetch details for.
    :return: list of detailed summary for each epoch.
    :rtype: list
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            if epoch_no is None:
                info = requests.get(self.EPOCH_INFO_URL, timeout=timeout)
                print(self.EPOCH_INFO_URL)
                info = json.loads(info.content)
            else:
                info = requests.get(f"{self.EPOCH_INFO_URL}?_epoch_no={epoch_no}", timeout=timeout)
                print(self.EPOCH_INFO_URL)
                info = json.loads(info.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return info


def get_epoch_params(self, epoch_no=None):
    """
    Get the protocol parameters for specific epoch, returns information about all epochs
    if no epoch specified.

    :param int epoch_no: epoch number to fetch details for.
    :return: list of protocol parameters for each epoch.
    :rtype: list
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            if epoch_no is None:
                info = requests.get(self.EPOCH_PARAMS_URL, timeout=timeout)
                info = json.loads(info.content)
            else:
                info = requests.get(f"{self.EPOCH_PARAMS_URL}?_epoch_no={epoch_no}", timeout=timeout)
                info = json.loads(info.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return info


def get_epoch_block_protocols(self, epoch_no=None):
    """
    Get the information about block protocol distribution in epoch

    :param int epoch_no: epoch number to fetch details for.
    :return: list of distinct block protocol versions counts in epoch
    :rtype: list
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            if epoch_no is None:
                info = requests.get(self.EPOCH_BLOCKS_URL, timeout=10)
                info = json.loads(info.content)
            else:
                info = requests.get(f"{self.EPOCH_BLOCKS_URL}?_epoch_no={epoch_no}", timeout=timeout)
                info = json.loads(info.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return info



#!/usr/bin/env python
"""
Provides all pool functions
"""
import json
import requests
from .environment import BASE_TIMEOUT, LIMIT_TIMEOUT


def get_pool_list(self, content_range="0-999"):
    """
    Get a list of all currently registered/retiring (not retired) pools.

    :param str range: paginated content range, up to  1000 records.
    :return: list of all registered/retiring pools.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            custom_headers = {"Range": str(content_range)}
            pool_list = requests.get(self.POOL_LIST_URL, headers = custom_headers, timeout=timeout)
            pool_list = json.loads(pool_list.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return pool_list


def get_pool_info(self, *args):
    """
    Get current pool status and details for a specified pool.

    :param str args: pool IDs in bech32 format (pool1...)
    :return: list of pool information.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            get_format = {"_pool_bech32_ids": [args] }
            pool_list = requests.post(self.POOL_INFO_URL, json = get_format, timeout=timeout)
            pool_list  = json.loads(pool_list.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return pool_list


def get_pool_stake_snapshot(self, pool_bech32):
    """
    Returns Mark, Set and Go stake snapshots for the selected pool, useful for leaderlog calculation

    :param str pool_bech32: Pool IDs in bech32 format (pool1...)
    :return: Array of pool stake information for 3 snapshots
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            snapshot = requests.get(self.POOL_STAKE_SNAPSHOT + pool_bech32, timeout=timeout)
            snapshot  = json.loads(snapshot.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return snapshot


def get_pool_delegators(self, pool_bech32, content_range="0-999"):
    """
    Return information about live delegators for a given pool.

    :param str pool_bech32: pool IDs in bech32 format (pool1...).
    :param str epoch_no: epoch number to get info (current if omitted).
    :return: list of pool delegators information.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            custom_headers = {"Range": str(content_range)}
            info = requests.get(self.POOL_DELEGATORS_URL + pool_bech32, headers = custom_headers, timeout=timeout)
            info = json.loads(info.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return info


def get_pool_delegators_history(self, pool_bech32, epoch_no=None):
    """
    Return information about active delegators (incl. history) for a given pool and epoch number \
    (all epochs if not specified).

    :param str pool_bech32: pool IDs in bech32 format (pool1...).
    :param str epoch_no: epoch number to get info (current if omitted).
    :return: list of pool delegators information.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            if epoch_no is None:
                info = requests.get(self.POOL_DELEGATORS_HISTORY_URL + pool_bech32, timeout=timeout)
                info = json.loads(info.content)
            else:
                info = requests.get(f"{self.POOL_DELEGATORS_HISTORY_URL}{pool_bech32}&_epoch_no={epoch_no}", timeout=timeout)
                info = json.loads(info.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return info


def get_pool_blocks(self, pool_bech32, epoch_no=None):
    """
    Return information about blocks minted by a given pool for all epochs (or _epoch_no if provided)

    :param str pool_bech32: pool IDs in bech32 format (pool1...).
    :param str epoch_no: epoch number to get info (from the beginning if omitted).
    :return: list of blocks created by pool.
    :rtype: list.s
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            if epoch_no is None:
                info = requests.get(self.POOL_BLOCKS_URL + pool_bech32, timeout=timeout)
                info = json.loads(info.content)
            else:
                info = requests.get(f"{self.POOL_BLOCKS_URL}{pool_bech32}&_epoch_no={epoch_no}", timeout=timeout)
                info = json.loads(info.content)
            break
        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return info


def get_pool_history(self, pool_bech32, epoch_no="history"):
    """
    Return information about pool stake, block and reward history in a given epoch _epoch_no \
    (or all epochs that pool existed for, in descending order if no _epoch_no was provided)

    :param str pool_bech32: pool IDs in bech32 format (pool1...).
    :param str epoch_no: epoch number to get info (from the beginning if omitted).
    :return: list of blocks created by pool.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            if epoch_no == "history":
                info = requests.get(f"{self.POOL_HISTORY_URL}{pool_bech32}", timeout=timeout)
                info = json.loads(info.content)
            else:
                info = requests.get(f"{self.POOL_HISTORY_URL}{pool_bech32}&_epoch_no={epoch_no}", timeout=timeout)
                info = json.loads(info.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return info


def get_pool_updates(self, pool_bech32=None):
    """
    Get all pool updates for all pools or only updates for specific pool if specified.

    :param str pool_bech32: pool IDs in bech32 format (pool1...).
    :return: list of historical pool updates.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            if pool_bech32 is None:
                pool_list = requests.get(self.POOL_UPDATES_URL, timeout=timeout)
                pool_list  = json.loads(pool_list.content)
            else:
                pool_list = requests.get(f"{self.POOL_UPDATES_URL}?_pool_bech32={pool_bech32}", timeout=timeout)
                pool_list  = json.loads(pool_list.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return pool_list


def get_pool_relays(self, content_range="0-999"):
    """
    Get a list of registered relays for all currently registered/retiring (not retired) pools.

    :param str range: paginated content range, up to  1000 records.
    :return: list of pool relay information.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            custom_headers = {"Range": str(content_range)}
            pool_list = requests.get(self.POOL_RELAYS_URL, headers = custom_headers, timeout=timeout)
            pool_list  = json.loads(pool_list.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return pool_list


def get_pool_metadata(self, *args):
    """
    Get Metadata (on & off-chain) for all currently registered/retiring (not retired) pools.

    :param str args: pool IDs in bech32 format (pool1...).
    :return: list of pool metadata.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            if len(args) == 0:
                pool_list = requests.post(self.POOL_METADATA_URL, timeout=timeout)
                pool_list  = json.loads(pool_list.content)
            else:
                get_format = {"_pool_bech32_ids": [args]}
                pool_list = requests.post(self.POOL_METADATA_URL, json = get_format, timeout=timeout)
                pool_list  = json.loads(pool_list.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return pool_list



#!/usr/bin/env python
"""
Provides all urls used in the library
"""

class URLs:
    # imported like class methods
    from .epoch import get_epoch_info, get_epoch_params, get_epoch_block_protocols
    from .network import get_tip, get_genesis, get_totals
    from .block import get_blocks, get_block_info, get_block_txs
    from .address import get_address_info, get_address_txs, get_address_assets, get_credential_txs
    from .account import get_account_info, get_account_info_cached, get_account_list, get_account_rewards, get_account_updates, get_account_addresses, get_account_assets, get_account_history, get_account_assets_2
    from .asset import get_asset_list, get_asset_address_list, get_asset_info, get_asset_history, get_asset_policy_info, get_asset_summary, get_asset_txs
    from .pool import get_pool_list, get_pool_info, get_pool_stake_snapshot, get_pool_delegators, get_pool_delegators_history, get_pool_blocks, get_pool_history, get_pool_updates, get_pool_relays, get_pool_metadata
    from .scripts import get_native_script_list, get_plutus_script_list, get_script_redeemers, get_datum_info
    from .transactions import get_tx_info, get_tx_utxos, get_tx_metadata, get_tx_metalabels, submit_tx, get_tx_status


    def __init__(self, url='https://api.koios.rest/api/v0/', network='mainnet'):
        
        self.url = url
        self.network = network
        
        # change subdomain to network name then change the rest of urls to use the new subdomain
        if self.network == 'testnet':
            # replace mainnet subdomain with "testnet" subdomain
            self.url = self.url.replace(self.url.split('.')[0], self.network)
            # add https:// to the url
            self.url = 'https://' + self.url
        elif self.network == 'mainnet':
            self.url = url


        # Network URLs
        self.TIP_URL = self.url + "tip"
        self.GENESIS_URL = self.url + "genesis"
        self.TOTALS_URL = self.url + "totals"
        # Epoch URLs
        self.EPOCH_INFO_URL = self.url + "epoch_info"
        self.EPOCH_PARAMS_URL = self.url + "epoch_params"
        self.EPOCH_BLOCKS_URL = self.url + "epoch_block_protocols"
        # Block URLs
        self.BLOCKS_URL = self.url + "blocks"
        self.BLOCK_INFO_URL = self.url + "block_info"
        self.BLOCK_TXS_URL = self.url + "block_txs"
        # Transaction URLs
        self.TX_INFO_URL = self.url + "tx_info"
        self.TX_UTXOS_URL = self.url + "tx_utxos"
        self.TX_METADATA_URL = self.url + "tx_metadata"
        self.TX_METALABELS_URL = self.url + "tx_metalabels"
        self.SUBMIT_TX_URL = self.url + "submittx"
        self.TX_STATUS_URL = self.url + "tx_status"
        # Address URLs
        self.ADDRESS_INFO_URL = self.url + "address_info"
        self.ADDRESS_TXS_URL = self.url + "address_txs"
        self.ADDRESS_ASSETS_URL = self.url + "address_assets"
        self.CREDENTIAL_TXS_URL = self.url + "credential_txs"
        # Account URLs
        self.ACCOUNT_LIST_URL = self.url + "account_list?offset="
        self.ACCOUNT_INFO_URL = self.url + "account_info"
        self.ACCOUNT_INFO_URL_CACHED = self.url + "account_info_cached"
        self.ACCOUNT_REWARDS_URL = self.url + "account_rewards"
        self.ACCOUNT_UPDATES_URL = self.url + "account_updates"
        self.ACCOUNT_ADDRESSES_URL = self.url + "account_addresses"
        #self.ACCOUNT_ASSETS_URL = self.url + "account_assets?offset="
        self.ACCOUNT_ASSETS_URL = self.url + "account_assets"
        self.ACCOUNT_HISTORY_URL = self.url + "account_history"
        # Asset URLs
        self.ASSET_LIST_URL = self.url + "asset_list"
        self.ASSET_ADDRESS_LIST_URL = self.url +  "asset_address_list?_asset_policy="
        self.ASSET_INFO_URL = self.url + "asset_info?_asset_policy="
        self.ASSET_HISTORY_URL = self.url + "asset_history?_asset_policy="
        self.ASSET_POLICY_INFO_URL = self.url + "asset_policy_info?_asset_policy="
        self.ASSET_SUMMARY_URL = self.url + "asset_summary?_asset_policy="
        self.ASSET_TXS_URL = self.url + "asset_txs?_asset_policy="

        # Pool URLs
        self.POOL_LIST_URL = self.url + "pool_list"
        self.POOL_INFO_URL = self.url + "pool_info"
        self.POOL_STAKE_SNAPSHOT = self.url + "pool_stake_snapshot?_pool_bech32="
        self.POOL_DELEGATORS_URL = self.url + "pool_delegators?_pool_bech32="
        self.POOL_DELEGATORS_HISTORY_URL = self.url + "pool_delegators_history?_pool_bech32="
        self.POOL_BLOCKS_URL = self.url + "pool_blocks?_pool_bech32="
        self.POOL_HISTORY_URL = self.url + "pool_history?_pool_bech32="
        self.POOL_UPDATES_URL = self.url + "pool_updates"
        self.POOL_RELAYS_URL = self.url + "pool_relays"
        self.POOL_METADATA_URL = self.url + "pool_metadata"

        # Scripts URLs
        self.NATIVE_SCRIPT_LIST_URL = self.url + "native_script_list"
        self.PLUTUS_SCRIPT_LIST_URL = self.url + "plutus_script_list"
        self.SCRIPT_REDEEMERS_URL = self.url + "script_redeemers?_script_hash="
        self.DATUM_INFO_URL = self.url + "datum_info"



#!/usr/bin/env python
"""
Provides all asset functions
"""
import json
import requests
from .environment import BASE_TIMEOUT, LIMIT_TIMEOUT


def get_asset_list(self, content_range="0-999"):
    """
    Get the list of all native assets (paginated)

    :return: list with all asset list.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            custom_headers = {"Range": str(content_range)}
            asset_list = requests.get(self.ASSET_LIST_URL, headers = custom_headers, timeout=timeout)
            asset_list = json.loads(asset_list.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return asset_list


def get_asset_address_list(self, asset_policy, asset_name, content_range="0-999"):
    """
    Get the list of all addresses holding a given asset.

    :param str asset_policy: asset Policy ID in hexadecimal format (hex).
    :param str asset_name: string with Asset Name in hexadecimal format (hex).
    :return: list of all addresses.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            custom_headers = {"Range": str(content_range)}
            info = requests.get(f"{self.ASSET_ADDRESS_LIST_URL}{asset_policy}&_asset_name={asset_name}", \
                headers = custom_headers, timeout=timeout)
            info = json.loads(info.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return info


def get_asset_info(self, asset_policy, asset_name):
    """
    Get the information of an asset including first minting & token registry metadata.

    :param str asset_policy: asset Policy ID in hexadecimal format (hex).
    :param str asset_name: string with Asset Name in hexadecimal format (hex).
    :return: list of all asset info.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            info = requests.get(f"{self.ASSET_INFO_URL}{asset_policy}&_asset_name={asset_name}", timeout=timeout)
            info = json.loads(info.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return info


def get_asset_history(self, asset_policy, asset_name):
    """
    Get the mint/burn history of an asset.

    :param str asset_policy: asset Policy ID in hexadecimal format (hex).
    :param str asset_name: string with Asset Name in hexadecimal format (hex).
    :return: list of asset mint/burn history.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            history = requests.get(f"{self.ASSET_HISTORY_URL}{asset_policy}&_asset_name={asset_name}", timeout=timeout)
            history = json.loads(history.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return history


def get_asset_policy_info(self, asset_policy):
    """
    Get the information for all assets under the same policy.

    :param str asset_policy: asset Policy ID in hexadecimal format (hex).
    :return: list of all mint/burn transactions for an asset
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            info = requests.get(f"{self.ASSET_POLICY_INFO_URL}{asset_policy}", timeout=timeout)
            info = json.loads(info.content)
            break
        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return info


def get_asset_summary(self, asset_policy, asset_name):
    """
    Get the summary of an asset (total transactions exclude minting/total wallets include only
    wallets with asset balance).

    :param str asset_policy: asset Policy ID in hexadecimal format (hex).
    :param str asset_name: string with Asset Name in hexadecimal format (hex).
    :return: list of asset summary information.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            summary = requests.get(f"{self.ASSET_SUMMARY_URL}{asset_policy}&_asset_name={asset_name}", timeout=timeout)
            summary = json.loads(summary.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return summary


def get_asset_txs(self, asset_policy, asset_name, after_block_height=0):
    """
    Get the list of all asset transaction hashes (newest first).

    :param str asset_policy: asset Policy ID in hexadecimal format (hex).
    :param str asset_name: string with Asset Name in hexadecimal format (hex).
    :param int after_block_height: Block height for specifying time delta, if not data start from 0
    :return: list of all asset hashes transactions.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            txs = requests.get(f"{self.ASSET_TXS_URL}{asset_policy}&_asset_name={asset_name}&_after_block_height={after_block_height}", timeout=timeout)
            txs = json.loads(txs.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return txs
    


#!/usr/bin/env python
"""
Provides all enviroment variables and constants used in the library
"""

# Timing
LIMIT_TIMEOUT= 60
BASE_TIMEOUT= 10
SLEEP_TIME= 10
OFFSET= 0
RETRYING_TIME= 1
LIMIT_RETRYING_TIMES= 10

# Exceptions Functions
def handle_timeout_exceptions(timeout, timeout_error):
    """
    Handle the timeout exceptions
    """
    print(f"Exception: {timeout_error}")
    if timeout < LIMIT_TIMEOUT:
        timeout= timeout + 10
    else:
        print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
        return
    print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")


#!/usr/bin/env python
"""
Provides all account functions
"""
import json
from time import sleep
import requests
from .environment import BASE_TIMEOUT, LIMIT_TIMEOUT, SLEEP_TIME, OFFSET, RETRYING_TIME, LIMIT_RETRYING_TIMES

def get_account_list(self, content_range="0-999"):
    """
    Get a list of all accounts.

    :return: string list of account (stake address: stake1...  bech32 format) IDs.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            custom_headers = {"Range": str(content_range)}
            address_list = requests.get(self.ACCOUNT_LIST_URL, headers = custom_headers, timeout=timeout)
            address_list = json.loads(address_list.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return address_list


def get_account_info(self, *args):
    """
    Get the account information for given stake addresses (accounts).

    :param str args: staking address/es in bech32 format (stake1...).
    :return: list with all address data.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            get_format = {"_stake_addresses": [args] }
            accounts_info = requests.post(self.ACCOUNT_INFO_URL, json= get_format , timeout=timeout)
            accounts_info = json.loads(accounts_info.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return accounts_info


def get_account_info_cached(self, *args):
    """
    Get the account information for given stake addresses (accounts).

    :param str args: staking address/es in bech32 format (stake1...).
    :return: list with all address data.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            get_format = {"_stake_addresses": [args] }
            accounts_info = requests.post(self.ACCOUNT_INFO_URL_CACHED, json= get_format , timeout=timeout)
            accounts_info = json.loads(accounts_info.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return accounts_info


def get_account_rewards(self, *args):
    """
    Get the full rewards history (including MIR) for given stake addresses (accounts).

    :param str args: Cardano staking address (reward account) in bech32 format (stake1...)
    :param int args: Epoch Number, has to be last parameter (optional).
    return: list with all account rewards.
    :rtype: list.
    """
    epoch = args[len(args)-1]
    timeout = BASE_TIMEOUT

    while True:
        try:
            if not isinstance(epoch, int):
                get_format = {"_stake_addresses": [args] }
                rewards = requests.post(self.ACCOUNT_REWARDS_URL, json= get_format , timeout=timeout)
                rewards = json.loads(rewards.content)
            else:
                get_format = {"_stake_addresses": [args], "_epoch_no": epoch}
                rewards = requests.post(self.ACCOUNT_REWARDS_URL, json= get_format , timeout=timeout)
                rewards = json.loads(rewards.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return rewards


def get_account_updates(self, *args):
    """
    Get the account updates (registration, deregistration, delegation and withdrawals) for given \
    stake addresses (accounts)

    :param str args: staking address/es in bech32 format (stake1...)
    :return: list with all account updates.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT
    while True:
        try:
            get_format = {"_stake_addresses": [args]}
            updates = requests.post(self.ACCOUNT_UPDATES_URL, json= get_format, timeout=timeout)
            updates = json.loads(updates.content)
            break
        
        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return updates


def get_account_addresses(self, *args):
    """
    Get all addresses associated with given staking accounts.
    :param str args: staking address/es in bech32 format (stake1...)
    :return: list with all account addresses.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT
    while True:
        try:
            get_format = {"_stake_addresses": [args]}
            addresses = requests.post(self.ACCOUNT_ADDRESSES_URL, json=get_format, timeout=timeout)
            addresses = json.loads(addresses.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return addresses


def get_account_assets(self, *args):
    """
    Get the native asset balance of given accounts.
    :param str args: staking address/es in bech32 format (stake1...)
    :return: list with all account assets.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT
    retriyng_time = RETRYING_TIME

    while True:
        try:
            get_format = {"_stake_addresses": [args]}
            assets = requests.post(self.ACCOUNT_ASSETS_URL, json= get_format, timeout=timeout)
            assets = json.loads(assets.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

        except json.decoder.JSONDecodeError as decode_error:
            print(f"Exception Decode: Payload too heavy. {decode_error}")
            sleep(SLEEP_TIME)
            retriyng_time += 1
            print(f"Retriyng one more time...({retriyng_time} times)")
            if retriyng_time >= LIMIT_RETRYING_TIMES:
                print("Reached limit of attempts")
                break

    return assets

## Alternative to Paginate all list automately
def get_account_assets_2(self, *args):
    """
    Get the native asset balance of given accounts.
    :param str args: staking address/es in bech32 format (stake1...)
    :return: list with all account assets.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT
    offset= OFFSET
    retriyng_time = RETRYING_TIME
    total_assets= []

    while True:
        while True:
            try:
                get_format = {"_stake_addresses": [args]}
                assets = requests.post(self.ACCOUNT_ASSETS_URL + str(offset), json= get_format, timeout=timeout)
                assets = json.loads(assets.content)
                break

            except requests.exceptions.ReadTimeout as timeout_error:
                print(f"Exception: {timeout_error}")
                if timeout < LIMIT_TIMEOUT:
                    timeout= timeout + 10
                else:
                    print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                    break
                print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

            except json.decoder.JSONDecodeError as decode_error:
                print(f"Exception Decode: Payload too heavy. {decode_error}")
                sleep(SLEEP_TIME)
                retriyng_time += 1
                print(f"Retriyng one more time...({retriyng_time} times)")
                if retriyng_time >= LIMIT_RETRYING_TIMES:
                    print("Reached limit of attempts")
                    break

        total_assets += assets
        if len(total_assets) < 1000:
            break
        offset += len(total_assets)

    return total_assets


def get_account_history(self, *args):
    """
    Get the staking history of given stake addresses (accounts).

    :param str address: staking address in bech32 format (stake1...)
    return: list with all account history.
    :rtype: list.
    """
    epoch = args[len(args)-1]
    timeout = BASE_TIMEOUT

    while True:
        try:
            if not isinstance(epoch, int):
                get_format = {"_stake_addresses": [args] }
                history = requests.post(self.ACCOUNT_HISTORY_URL, json= get_format , timeout=timeout)
                history = json.loads(history.content)
            else:
                get_format = {"_stake_addresses": [args], "_epoch_no": epoch}
                history = requests.post(self.ACCOUNT_HISTORY_URL, json= get_format , timeout=timeout)
                history = json.loads(history.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return history



#!/usr/bin/env python
"""
Provides all network functions
"""
import json
import requests
from .environment import BASE_TIMEOUT, LIMIT_TIMEOUT

def get_tip(self):
    """
    Get the tip info about the latest block seen by chain.

    :return: list of block summary (limit+paginated).
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            tip = requests.get(self.TIP_URL, timeout=timeout)
            tip = json.loads(tip.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return tip


def get_genesis(self):
    """
    Get the Genesis parameters used to start specific era on chain.

    :return: list of genesis parameters used to start each era on chain.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            genesis = requests.get(self.GENESIS_URL, timeout=timeout)
            genesis = json.loads(genesis.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return genesis


def get_totals(self, epoch_no=None):
    """
    Get the circulating utxo, treasury, rewards, supply and reserves in lovelace for specified
    epoch, all epochs if empty.

    :param int epoch_no: Epoch Number to fetch details for.
    :return: list of of supply/reserves/utxo/fees/treasury stats.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            if epoch_no is None:
                totals = requests.get(self.TOTALS_URL, timeout=timeout)
                totals = json.loads(totals.content)
            else:
                totals = requests.get(f"{self.TOTALS_URL}?_epoch_no={epoch_no}", timeout=timeout)
                totals = json.loads(totals.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return totals
    


#!/usr/bin/env python
"""
Provides all block functions
"""
import json
import requests
from .environment import BASE_TIMEOUT, LIMIT_TIMEOUT

def get_blocks(self,content_range="0-999"):
    """
    Get summarised details about all blocks (paginated - latest first).

    :param str range: paginated content range, up to  1000 records.
    :return: list of all blocks.
    :rtype: list
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            custom_headers = {"Range": str(content_range)}
            blocks = requests.get(self.BLOCKS_URL, headers = custom_headers, timeout=timeout)
            blocks = json.loads(blocks.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return blocks


def get_block_info(self,*block_hash):
    """
    Get detailed information about a specific block or blocks

    :param str block_hash: block/s hash ID.
    :return:  list of detailed block information.
    :rtype: list
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            get_format = {"_block_hashes":[block_hash]}
            block = requests.post(self.BLOCK_INFO_URL, json = get_format, timeout=timeout)
            block = json.loads(block.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return block


def get_block_txs(self,*block_hash):
    """
    Get a list of all transactions included in a provided block.

    :param str block_hash: block hash ID.
    :return: list of transactions hashes.
    :rtype: list
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            get_format = {"_block_hashes":[block_hash]}
            txs = requests.post(self.BLOCK_TXS_URL, json = get_format, timeout=timeout)
            txs = json.loads(txs.content)
            break
        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return txs



#!/usr/bin/env python
"""
Provides all scripts functions
"""
import json
from time import sleep
import requests
from .environment import BASE_TIMEOUT, LIMIT_TIMEOUT, SLEEP_TIME


def get_native_script_list(self, content_range="0-999"):
    """
    Get list of all existing native script hashes along with their creation transaction hashes

    :param str range: paginated content range, up to  1000 records.
    return: list of native script and creation tx hash pairs.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            custom_headers = {"Range": str(content_range)}
            get_format = requests.post(self.NATIVE_SCRIPT_LIST_URL, headers = custom_headers, timeout=timeout)
            get_format = json.loads(get_format.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

        except json.decoder.JSONDecodeError as decode_error:
            print(f"Exception Decode: {decode_error}")
            sleep(SLEEP_TIME)
            print("Retriyng one more time...")

    return get_format


def get_plutus_script_list(self, content_range="0-999"):
    """
    Get list of all existing Plutus script hashes along with their creation transaction hashes.

    :param str range: paginated content range, up to  1000 records.
    return: list of Plutus script and creation tx hash pairs.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            custom_headers = {"Range": str(content_range)}
            get_format = requests.post(self.PLUTUS_SCRIPT_LIST_URL, headers = custom_headers, timeout=timeout)
            get_format = json.loads(get_format.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

        except json.decoder.JSONDecodeError as decode_error:
            print(f"Exception Decode: {decode_error}")
            sleep(SLEEP_TIME)
            print("Retriyng one more time...")

    return get_format


def get_script_redeemers(self, script_hash):
    """
    Get list of all redeemers for a given script hash.

    :params string script_hash: script hash in hexadecimal format (hex) to search and read data.
    :return: list of all redeemers for a given script hash.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            query = requests.get(self.SCRIPT_REDEEMERS_URL + script_hash, timeout=timeout)
            query  = json.loads(query.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return query


def get_datum_info(self, *datum_hash):
    '''
    Get list of datum information for given datum hashes

    :params string datum_hash: The Hash of the Plutus Data.
    :return: the actual data in json form.
    :rtype: list.
    '''
    timeout = BASE_TIMEOUT

    while True:
        try:
            custom_headers = {"_datum_hashes": [datum_hash]}
            get_format = requests.post(self.DATUM_INFO_URL, json = custom_headers, timeout=timeout)
            get_format = json.loads(get_format.content)
            break
        
        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return get_format


#!/usr/bin/env python
"""
Provides all transactions functions
"""
import json
import requests
from .environment import BASE_TIMEOUT, LIMIT_TIMEOUT

def get_tx_info(self, *args):
    """
    Get detailed information about transaction(s).

    :param list tx_hash: list of transaction(s) hash to search and read data.
    :return: list of all info about transaction(s).
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            get_format = {"_tx_hashes": [args]}
            tx_info = requests.post(self.TX_INFO_URL, json = get_format, timeout=timeout)
            tx_info  = json.loads(tx_info.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return tx_info


def get_tx_utxos(self, *args):
    """
    Get UTxO set (inputs/outputs) of transactions.

    :param list tx_hash: list of transaction(s) hash to search and read utxos data.
    :return: all info about utxos in transaction(s)
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            get_format = {"_tx_hashes": [args]}
            tx_utxos = requests.post(self.TX_UTXOS_URL, json = get_format, timeout=timeout)
            tx_utxos  = json.loads(tx_utxos.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return tx_utxos


def get_tx_metadata(self, *args):
    """
    Get metadata information (if any) for given transaction(s).

    :param list tx_hash: list strings of transaction(s) hash to search and read utxos data.
    :return: list of all info about utxos in transaction(s)
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            get_format = {"_tx_hashes": [args]}
            tx_metadata = requests.post(self.TX_METADATA_URL, json = get_format, timeout=timeout)
            tx_metadata  = json.loads(tx_metadata.content)
            break
        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return tx_metadata


def get_tx_metalabels(self, content_range="0-999"):
    """
    Get a list of all transaction metalabels.

    :param str range: paginated content range, up to  1000 records.
    :return: list of metalabels transactions
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            custom_headers = {"Range": str(content_range)}
            tx_metalabels = requests.get(self.TX_METALABELS_URL, headers \
            = custom_headers, timeout=timeout)
            tx_metalabels  = json.loads(tx_metalabels.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return tx_metalabels


def submit_tx(self, file):
    """
    Submit an already serialized transaction to the network. You have to serialized the transaction
    file with: xxd -r -p <<< $(jq .cborHex signed.txt) > signed.cbor

    :param file: a file with raw binary serialized transaction on the file-system.
    :return: hex transaction ID (if is successful )
    """
    with open(file, "rb") as cbor_tx:
        cbor_tx = cbor_tx.read()
    cbor_header = {'Content-Type': 'application/cbor'}
    submit = requests.post(self.SUBMIT_TX_URL, headers = cbor_header, \
        data = cbor_tx, timeout=25)
    submit  = json.loads(submit.content)
    return submit


def get_tx_status(self, *args):
    """
    Get the number of block confirmations for a given transaction hash list.

    :param list tx_hash: list of transaction(s) hash to search and read utxos data.
    :return: list of all info about utxos in transaction(s)
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            get_format = {"_tx_hashes": [args]}
            tx_status = requests.post(self.TX_STATUS_URL, json = get_format, timeout=timeout)
            tx_status  = json.loads(tx_status.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return tx_status
    


"""
Provides all modules
"""
from .block import *
from .epoch import *
from .network import *
from .transactions import *
from .address import *
from .account import *
from .asset import *
from .pool import *
from .scripts import *
from .urls import *
from .environment import *



#!/usr/bin/env python3

import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent
LONG_DESCRIPTION = (HERE / "README.md").read_text(encoding='utf-8')

setup(
    version = '1.1.0',
    name = 'koios-python',
    author = 'Quixote Stake Pool',
    author_email = 'quixotepool@proton.me',
    url = 'https://github.com/QuixoteSystems',
    license = 'MIT',
    description = 'Python wrapper Library using Koios API for accessing information stored on the Cardano Blockchain',
    long_description= LONG_DESCRIPTION,
    long_description_content_type= 'text/markdown',
    keywords = ['koios', 'blockchain', 'cardano', 'API', 'REST', 'RESTful'],
    include_package_data = True,
    packages = find_packages(include=['koios_python']),
    install_requieres = ["requests"],
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        'Intended Audience :: Developers',

        'Topic :: Software Development :: Build Tools',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3 :: Only',
    ],
)



#!/usr/bin/env python
"""
Provides all address functions
"""
import json
import requests
from .environment import BASE_TIMEOUT, LIMIT_TIMEOUT

def get_address_info(self, *args):
    """
    Get address info - balance, associated stake address (if any) and UTxO set.

    :param str address: wallet used public address(es).
    return: list with data of this used public address.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            get_format = {"_addresses": [args] }
            addresses = requests.post(self.ADDRESS_INFO_URL, json= get_format, timeout=timeout)
            addresses = json.loads(addresses.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return addresses


def get_address_txs(self, address_tx, after_block=0):
    """
    Get the transaction hash list of input address array, optionally filtering after specified
    block height (inclusive)

    :param tx_hash: list or single transaction hash to search and read utxos data
    :param after_block: filtering after block (inclusive) defaul is 0, from the beginning
    :return: hash list of address transactions
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            get_format = {"_addresses": [address_tx], "_after_block_height": str(after_block)}
            hash_list = requests.post(self.ADDRESS_TXS_URL, json = get_format, timeout=timeout)
            hash_list  = json.loads(hash_list.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return hash_list


def get_address_assets(self, *args):
    """
    Get the list of all the assets (policy, name and quantity) for a given address.

    :param str address: wallet used public address
    return: list of all the assets
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            get_format = {"_addresses": [args] }
            addresses = requests.post(self.ADDRESS_ASSETS_URL, json= get_format , timeout=timeout)
            addresses = json.loads(addresses.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return addresses


def get_credential_txs(self, payment_credentials, after_block=0):
    """
    Get the transaction hash list of input payment credential array (stake key), optionally
    filtering after specified block height (inclusive).

    :param str payment_credentials: list address payment credential array (stake key)
    :param int after_block: filtering after block (inclusive) defaul is 0, from the beginning
    :return: hash list of address transactions.
    :rtype: list.
    """
    timeout = BASE_TIMEOUT

    while True:
        try:
            get_format = {"_payment_credentials":[payment_credentials], "_after_block_height": after_block}
            hash_list = requests.post(self.CREDENTIAL_TXS_URL, json = get_format, timeout=timeout)
            hash_list  = json.loads(hash_list.content)
            break

        except requests.exceptions.ReadTimeout as timeout_error:
            print(f"Exception: {timeout_error}")
            if timeout < LIMIT_TIMEOUT:
                timeout= timeout + 10
            else:
                print(f"Reach Limit Timeout= {LIMIT_TIMEOUT} seconds")
                break
            print(f"Retriyng with longer timeout: Total Timeout= {timeout}s")

    return hash_list
